@@ -1631,7 +1631,10 @@ public:
       return mysql_data_home;
     if (is_absolute_path(path))
       return path;
-    return buf.assign(mysql_data_home).append(path).c_str();
+    buf.assign(mysql_data_home);
+    if (!buf.empty() && buf.back() != '/' IF_WIN(&& buf.back() != '\\', ))
+      buf.append("/");
+    return buf.append(path).c_str();
   }
 };
 
@@ -1755,7 +1758,7 @@ copy_back()
 
 	for (uint i = 1; i <= TRX_SYS_MAX_UNDO_SPACES; i++) {
 		char filename[20];
-		sprintf(filename, "undo%03u", i);
+		snprintf(filename, sizeof(filename), "undo%03u", i);
 		if (!file_exists(filename)) {
 			break;
 		}
