@@ -148,7 +148,7 @@ xb_mysql_connect()
 	char mysql_port_str[std::numeric_limits<int>::digits10 + 3];
 	const char *user= opt_user ? opt_user : get_os_user();
 
-	sprintf(mysql_port_str, "%d", opt_port);
+	snprintf(mysql_port_str, sizeof(mysql_port_str), "%d", opt_port);
 
 	if (connection == NULL) {
 		msg("Failed to init MariaDB struct: %s.",
@@ -549,16 +549,15 @@ bool get_mysql_vars(MYSQL *connection)
   if (ignore_db_dirs)
     xb_load_list_string(ignore_db_dirs, ",", register_ignore_db_dirs_filter);
 
-  if (binlog_directory_var && *binlog_directory_var)
-  {
-    if (free_opt_binlog_directory)
-      my_free(const_cast<char *>(opt_binlog_directory));
-    opt_binlog_directory= my_strdup(PSI_NOT_INSTRUMENTED, binlog_directory_var,
-                                    MYF(MY_FAE));
-    free_opt_binlog_directory= true;
-  }
+  if (free_opt_binlog_directory)
+    my_free(const_cast<char *>(opt_binlog_directory));
+  opt_binlog_directory= my_strdup(PSI_NOT_INSTRUMENTED,
+                                  (binlog_directory_var ? binlog_directory_var : ""),
+                                  MYF(MY_FAE));
+  free_opt_binlog_directory= true;
 
 out:
+  free_mysql_variables(mysql_vars);
 
   return (ret);
 }
@@ -1526,7 +1525,9 @@ write_galera_info(ds_ctxt *datasink, MYSQL *connection)
       domain_id ? domain_id : domain_id55);
 
 cleanup:
+  free_mysql_variables(vars);
   free_mysql_variables(status);
+  free_mysql_variables(value);
 
   return(result);
 }
