@@ -568,7 +568,7 @@ read_cnf()
         if [ "$tmode" != 'DISABLED' ]; then
             if [ 0 -eq $encrypt -a -n "$tpem" -a -n "$tkey" ]
             then
-                encrypt=3 # enable cert/key SSL encyption
+                encrypt=3 # enable cert/key SSL encryption
                 # avoid CA verification if not set explicitly:
                 # nodes may happen to have different CA if self-generated,
                 # zeroing up tcert and tcap does the trick:
@@ -1006,14 +1006,14 @@ else
         if [ -n "$sstlogarchivedir" ]; then
             if [ ! -d "$sstlogarchivedir" ]; then
                 if ! mkdir -p "$sstlogarchivedir"; then
-                    sstlogarchivedir=""
                     wsrep_log_warning \
                         "Unable to create '$sstlogarchivedir' directory"
+                    sstlogarchivedir=""
                 fi
             elif [ ! -w "$sstlogarchivedir" ]; then
-                sstlogarchivedir=""
                 wsrep_log_warning \
-                    "The '$sstlogarchivedir' directory is not writtable"
+                    "The '$sstlogarchivedir' directory is not writable"
+                sstlogarchivedir=""
             fi
         fi
 
