@@ -1377,10 +1377,9 @@ Event_job_data::execute(THD *thd, bool drop)
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   if (event_sctx.change_security_context(thd, &definer_user, &definer_host,
                                          &dbname, &save_sctx) ||
-      mysql_change_db(thd, &dbname, false))
+      mysql_change_db(thd, dbname, FALSE))
   {
-    sql_print_error("Event Scheduler: "
-                    "[%s].[%s.%s] execution failed, "
+    sql_print_error("Event Scheduler: [%s].[%s.%s] execution failed, "
                     "failed to authenticate the user.",
                     definer.str, dbname.str, name.str);
     goto end;
@@ -1395,8 +1394,7 @@ Event_job_data::execute(THD *thd, bool drop)
       privilege is revoked from trigger definer,
       triggers are not executed.
     */
-    sql_print_error("Event Scheduler: "
-                    "[%s].[%s.%s] execution failed, "
+    sql_print_error("Event Scheduler: [%s].[%s.%s] execution failed, "
                     "user no longer has EVENT privilege.",
                     definer.str, dbname.str, name.str);
     goto end;
@@ -1476,8 +1474,9 @@ end:
       We must do it here since here we're under the right authentication
       ID of the event definer.
     */
-    sql_print_information("Event Scheduler: Dropping %s.%s",
-                          (const char *) dbname.str, (const char *) name.str);
+    if (global_system_variables.log_warnings > 2)
+      sql_print_information("Event Scheduler: Dropping %s.%s",
+                            dbname.str, name.str);
     /*
       Construct a query for the binary log, to ensure the event is dropped
       on the slave
