@@ -235,7 +235,8 @@ public:
   */
   virtual void reset_value_registration() {}
   enum_parsing_place place() { return parsing_place; }
-  bool walk(Item_processor processor, bool walk_subquery, void *arg) override;
+  bool walk(Item_processor processor, void *arg,
+            item_walk_flags flags) override;
   bool unknown_splocal_processor(void *arg) override;
   bool mark_as_eliminated_processor(void *arg) override;
   bool eliminate_subselect_processor(void *arg) override;
@@ -274,11 +275,12 @@ public:
   void register_as_with_rec_ref(With_element *with_elem);
   void init_expr_cache_tracker(THD *thd);
 
-  Item* do_build_clone(THD *thd) const override { return nullptr; }
-  Item *do_get_copy(THD *thd) const override { return 0; }
-
   st_select_lex *wrap_tvc_into_select(THD *thd, st_select_lex *tvc_sl);
 
+protected:
+  Item* deep_copy(THD *thd) const override { return nullptr; }
+  Item *shallow_copy(THD *thd) const override { return nullptr; }
+
   friend class select_result_interceptor;
   friend class Item_in_optimizer;
   friend bool Item_field::fix_fields(THD *, Item **);
@@ -764,10 +766,11 @@ public:
     DBUG_VOID_RETURN;
   }
 
-  bool walk(Item_processor processor, bool walk_subquery, void *arg) override
+  bool walk(Item_processor processor, void *arg,
+            item_walk_flags flags) override
   {
-    return left_expr->walk(processor, walk_subquery, arg) ||
-           Item_subselect::walk(processor, walk_subquery, arg);
+    return left_expr->walk(processor, arg, flags) ||
+           Item_subselect::walk(processor, arg, flags);
   }
 
   bool exists2in_processor(void *opt_arg __attribute__((unused))) override
@@ -789,6 +792,22 @@ public:
   Subq_materialization_tracker *get_materialization_tracker() const
   { return materialization_tracker; }
 
+  bool ora_join_processor(void *arg) override
+  {
+    if (left_expr->with_ora_join() && left_expr->cols() > 1)
+    {
+      // used in ROW operaton
+      my_error(ER_INVALID_USE_OF_ORA_JOIN_WRONG_FUNC, MYF(0));
+      return TRUE;
+    }
+    return FALSE;
+  }
+  bool add_maybe_null_after_ora_join_processor(void *arg) override
+  {
+    if (!maybe_null() && left_expr->maybe_null())
+      set_maybe_null();
+    return 0;
+  }
   friend class Item_ref_null_helper;
   friend class Item_is_not_null_test;
   friend class Item_in_optimizer;
