@@ -150,7 +150,8 @@ public:
     return name;
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_row_number>(thd, this); }
 };
 
@@ -225,7 +226,9 @@ public:
     }
     Item_sum_int::cleanup();
   }
-  Item *do_get_copy(THD *thd) const override
+
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_rank>(thd, this); }
 };
 
@@ -298,7 +301,9 @@ class Item_sum_dense_rank: public Item_sum_int
     }
     Item_sum_int::cleanup();
   }
-  Item *do_get_copy(THD *thd) const override
+
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_dense_rank>(thd, this); }
 };
 
@@ -317,7 +322,7 @@ class Item_sum_hybrid_simple : public Item_sum_hybrid
 
   bool add() override;
   bool fix_fields(THD *, Item **) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   void setup_hybrid(THD *thd, Item *item);
   double val_real() override;
   longlong val_int() override;
@@ -362,7 +367,8 @@ class Item_sum_first_value : public Item_sum_hybrid_simple
     return name;
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_first_value>(thd, this); }
 };
 
@@ -389,7 +395,8 @@ class Item_sum_last_value : public Item_sum_hybrid_simple
     return name;
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_last_value>(thd, this); }
 };
 
@@ -411,7 +418,8 @@ class Item_sum_nth_value : public Item_sum_hybrid_simple
     return name;
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_nth_value>(thd, this); }
 };
 
@@ -433,7 +441,8 @@ class Item_sum_lead : public Item_sum_hybrid_simple
     return name;
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_lead>(thd, this); }
 };
 
@@ -455,7 +464,8 @@ class Item_sum_lag : public Item_sum_hybrid_simple
     return name;
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_lag>(thd, this); }
 };
 
@@ -554,7 +564,7 @@ class Item_sum_percent_rank: public Item_sum_double,
   const Type_handler *type_handler() const override
   { return &type_handler_double; }
 
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals = 10;  // TODO-cvicentiu find out how many decimals the standard
                     // requires.
@@ -570,7 +580,8 @@ class Item_sum_percent_rank: public Item_sum_double,
     Partition_row_count::set_partition_row_count(count);
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_percent_rank>(thd, this); }
 
  private:
@@ -644,7 +655,7 @@ class Item_sum_cume_dist: public Item_sum_double,
   const Type_handler *type_handler() const override
   { return &type_handler_double; }
 
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals = 10;  // TODO-cvicentiu find out how many decimals the standard
                     // requires.
@@ -658,9 +669,9 @@ class Item_sum_cume_dist: public Item_sum_double,
     Partition_row_count::set_partition_row_count(count);
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_cume_dist>(thd, this); }
-
 };
 
 class Item_sum_ntile : public Item_sum_int,
@@ -735,7 +746,8 @@ class Item_sum_ntile : public Item_sum_int,
     Partition_row_count::set_partition_row_count(count);
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_ntile>(thd, this); }
 
  private:
@@ -885,7 +897,7 @@ public:
   const Type_handler *type_handler() const override
   {return Type_handler_hybrid_field_type::type_handler();}
 
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals = 10;  // TODO-cvicentiu find out how many decimals the standard
                     // requires.
@@ -899,8 +911,11 @@ public:
     Partition_row_count::set_partition_row_count(count);
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_percentile_disc>(thd, this); }
+
+public:
   void setup_window_func(THD *thd, Window_spec *window_spec) override;
   void setup_hybrid(THD *thd, Item *item);
   bool fix_fields(THD *thd, Item **ref) override;
@@ -1022,7 +1037,7 @@ public:
   }
   void update_field() override {}
 
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     decimals = 10;  // TODO-cvicentiu find out how many decimals the standard
                     // requires.
@@ -1036,8 +1051,11 @@ public:
     Partition_row_count::set_partition_row_count(count);
   }
 
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_sum_percentile_cont>(thd, this); }
+
+public:
   void setup_window_func(THD *thd, Window_spec *window_spec) override;
   void setup_hybrid(THD *thd, Item *item);
   bool fix_fields(THD *thd, Item **ref) override;
@@ -1188,7 +1206,7 @@ public:
 
   /*
     Computation functions.
-    TODO: consoder merging these with class Group_bound_tracker.
+    TODO: consider merging these with class Group_bound_tracker.
   */
   void setup_partition_border_check(THD *thd);
 
@@ -1373,7 +1391,7 @@ public:
   void split_sum_func(THD *thd, Ref_ptr_array ref_pointer_array,
                               List<Item> &fields, uint flags) override;
 
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     Type_std_attributes::set(window_func());
     return FALSE;
@@ -1391,8 +1409,8 @@ public:
   
   void print(String *str, enum_query_type query_type) override;
 
- Item *do_get_copy(THD *thd) const override { return 0; }
-
+protected:
+  Item *shallow_copy(THD *thd) const override { return nullptr; }
 };
 
 #endif /* ITEM_WINDOWFUNC_INCLUDED */
