@@ -176,8 +176,8 @@ public:
       }
     }
 
-    str->length(0);
-    str->set_charset(collation.collation);
+    // Make sure we never return {Ptr=nullptr, str_length=0}
+    str->copy("", 0, collation.collation);
     for (uint i=0 ; i < numnodes; i++)
     {
       if(active[i])
@@ -221,7 +221,8 @@ public:
     return { STRING_WITH_LEN("xpath_rootelement") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_rootelement>(thd, this); }
 };
 
@@ -237,7 +238,8 @@ public:
     return { STRING_WITH_LEN("xpath_union") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_union>(thd, this); }
 };
 
@@ -277,7 +279,8 @@ public:
     return { STRING_WITH_LEN("xpath_selfbyname") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_selfbyname>(thd, this); }
 };
 
@@ -294,7 +297,8 @@ public:
     return { STRING_WITH_LEN("xpath_childbyname") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_childbyname>(thd, this); }
 };
 
@@ -313,7 +317,8 @@ public:
     return { STRING_WITH_LEN("xpath_descendantbyname") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_descendantbyname>(thd, this); }
 };
 
@@ -332,7 +337,8 @@ public:
     return { STRING_WITH_LEN("xpath_ancestorbyname") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_ancestorbyname>(thd, this); }
 };
 
@@ -350,7 +356,8 @@ public:
     return { STRING_WITH_LEN("xpath_parentbyname") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_parentbyname>(thd, this); }
 };
 
@@ -367,7 +374,8 @@ public:
     return { STRING_WITH_LEN("xpath_attributebyname") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_attributebyname>(thd, this); }
 };
 
@@ -387,7 +395,8 @@ public:
     return { STRING_WITH_LEN("xpath_predicate") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_predicate>(thd, this); }
 };
 
@@ -403,7 +412,8 @@ public:
     return { STRING_WITH_LEN("xpath_elementbyindex") };
   }
   bool val_native(THD *thd, Native *nodeset) override;
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_func_elementbyindex>(thd, this); }
 };
 
@@ -434,7 +444,8 @@ public:
     }
     return args[0]->val_real() ? 1 : 0;
   }
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_xpath_cast_bool>(thd, this); }
 };
 
@@ -451,7 +462,8 @@ public:
     return { STRING_WITH_LEN("xpath_cast_number") };
   }
   double val_real() override { return args[0]->val_real(); }
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_xpath_cast_number>(thd, this); }
 };
 
@@ -471,7 +483,8 @@ public:
   }
   bool fix_length_and_dec(THD *thd) override
   { max_length= MAX_BLOB_WIDTH; return FALSE; }
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_context_cache>(thd, this); }
 };
 
@@ -495,7 +508,8 @@ public:
       return tmp_native_value.element(0).pos + 1;
     return 0;
   }
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_func_xpath_position>(thd, this); }
 };
 
@@ -521,7 +535,8 @@ public:
       return predicate_supplied_context_size;
     return tmp_native_value.elements();
   }
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_func_xpath_count>(thd, this); }
 };
 
@@ -569,7 +584,8 @@ public:
     }
     return sum;
   }
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_func_xpath_sum>(thd, this); }
 };
 
@@ -654,7 +670,8 @@ public:
     }
     return 0;
   }
-  Item *do_get_copy(THD *thd) const override
+protected:
+  Item *shallow_copy(THD *thd) const override
   { return get_item_copy<Item_nodeset_to_const_comparator>(thd, this); }
 };
 
@@ -3072,6 +3089,26 @@ String *Item_func_xml_extractvalue::val_str(String *str)
 }
 
 
+const Type_handler *Item_func_xml_update::xml_handler= NULL;
+
+
+bool Item_func_xml_update::fix_length_and_dec(THD *thd)
+{
+  static LEX_CSTRING name= {STRING_WITH_LEN("XMLTYPE")};
+
+  if (!xml_handler)
+    xml_handler= Type_handler::handler_by_name(thd, name);
+
+  return Item_xml_str_func::fix_length_and_dec(thd);
+}
+
+
+const Type_handler *Item_func_xml_update::type_handler() const
+{
+  return xml_handler ? xml_handler : Item_xml_str_func::type_handler();
+}
+
+
 bool Item_func_xml_update::collect_result(String *str,
                                           const MY_XML_NODE *cut,
                                           const String *replace)
