@@ -55,6 +55,7 @@ SYMBOL symbols[] = {
   { "<<",		SYM(SHIFT_LEFT)},
   { ">>",		SYM(SHIFT_RIGHT)},
   { "<=>",		SYM(EQUAL_SYM)},
+  { "=>",   SYM(ARROW_SYM)},
   { "ACCOUNT",		SYM(ACCOUNT_SYM)},
   { "ACTION",		SYM(ACTION)},
   { "ADD",		SYM(ADD)},
@@ -69,6 +70,7 @@ SYMBOL symbols[] = {
   { "ANALYZE",		SYM(ANALYZE_SYM)},
   { "AND",		SYM(AND_SYM)},
   { "ANY",              SYM(ANY_SYM)},
+  { "ARRAY",		SYM(ARRAY_SYM)},
   { "AS",		SYM(AS)},
   { "ASC",		SYM(ASC)},
   { "ASCII",		SYM(ASCII_SYM)},
@@ -149,13 +151,15 @@ SYMBOL symbols[] = {
   { "CONTEXT",    SYM(CONTEXT_SYM)},
   { "CONTINUE",         SYM(CONTINUE_MARIADB_SYM)},
   { "CONTRIBUTORS",     SYM(CONTRIBUTORS_SYM)},
+  { "CONVERSION",       SYM(CONVERSION_SYM)},
   { "CONVERT",		SYM(CONVERT_SYM)},
-  { "CPU",        SYM(CPU_SYM)},
+  { "CPU",              SYM(CPU_SYM)},
   { "CREATE",		SYM(CREATE)},
   { "CROSS",		SYM(CROSS)},
   { "CUBE",		SYM(CUBE_SYM)},
   { "CURRENT",          SYM(CURRENT_SYM)},
   { "CURRENT_DATE",	SYM(CURDATE)},
+  { "CURRENT_PATH",	SYM(CURRENT_PATH)},
   { "CURRENT_POS",      SYM(CURRENT_POS_SYM)},
   { "CURRENT_ROLE",	SYM(CURRENT_ROLE)},
   { "CURRENT_TIME",	SYM(CURTIME)},
@@ -441,6 +445,7 @@ SYMBOL symbols[] = {
   { "NUMBER",           SYM(NUMBER_MARIADB_SYM)},
   { "NUMERIC",		SYM(NUMERIC_SYM)},
   { "NVARCHAR",		SYM(NVARCHAR_SYM)},
+  { "OBJECT",		SYM(OBJECT_SYM)},
   { "OF",		SYM(OF_SYM)},
   { "OFFSET",		SYM(OFFSET_SYM)},
   { "OLD_PASSWORD",	SYM(OLD_PASSWORD_SYM)},
@@ -565,6 +570,7 @@ SYMBOL symbols[] = {
   { "ROW_NUMBER",	SYM(ROW_NUMBER_SYM)},
   { "RTREE",		SYM(RTREE_SYM)},
   { "SAVEPOINT",	SYM(SAVEPOINT_SYM)},
+  { "SCALAR",		SYM(SCALAR_SYM)},
   { "SCHEDULE",		SYM(SCHEDULE_SYM)},
   { "SCHEMA",		SYM(DATABASE)},
   { "SCHEMA_NAME",      SYM(SCHEMA_NAME_SYM)},
@@ -671,6 +677,7 @@ SYMBOL symbols[] = {
   { "TINYINT",		SYM(TINYINT)},
   { "TINYTEXT",		SYM(TINYTEXT)},
   { "TO",		SYM(TO_SYM)},
+  { "TO_DATE",		SYM(TO_DATE)},
   { "TRAILING",		SYM(TRAILING)},
   { "TRANSACTION",	SYM(TRANSACTION_SYM)},
   { "TRANSACTIONAL",	SYM(TRANSACTIONAL_SYM)},
@@ -780,6 +787,7 @@ SYMBOL sql_functions[] = {
   { "STDDEV",		SYM(STD_SYM)},
   { "STDDEV_POP",	SYM(STD_SYM)},
   { "STDDEV_SAMP",	SYM(STDDEV_SAMP_SYM)},
+  { "ST_COLLECT", SYM(ST_COLLECT_SYM)},
   { "SUBDATE",		SYM(SUBDATE_SYM)},
   { "SUBSTR",		SYM(SUBSTRING)},
   { "SUBSTRING",	SYM(SUBSTRING)},
