@@ -831,7 +831,9 @@ bool partition_info::vers_set_hist_part(THD *thd, uint *create_count)
     return 0;
   }
   else if (vers_info->interval.is_set() &&
-           vers_info->hist_part->range_value <= (longlong) thd->query_start())
+           /* Left boundary is closed */
+           vers_info->hist_part->range_value <= (longlong)thd->query_start() &&
+           vers_info->hist_part->range_value < TIMESTAMP_MAX_VALUE)
   {
     partition_element *next= NULL;
     bool error= true;
@@ -842,7 +844,8 @@ bool partition_info::vers_set_hist_part(THD *thd, uint *create_count)
     while ((next= it++) != vers_info->now_part)
     {
       vers_info->hist_part= next;
-      if (next->range_value > (longlong) thd->query_start())
+      /* Right boundary is open */
+      if ((longlong)thd->query_start() < next->range_value)
       {
         error= false;
         break;
