@@ -16,10 +16,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1335  USA */
 
-#ifdef USE_PRAGMA_INTERFACE
-#pragma interface			/* gcc class implementation */
-#endif
-
 #include "sql_class.h"
 #include "partition_element.h"
 #include "sql_partition.h"
@@ -36,23 +32,33 @@ struct st_ddl_log_memory_entry;
 
 #define MAX_PART_NAME_SIZE 8
 
-
 struct Vers_part_info : public Sql_alloc
 {
   Vers_part_info() :
     limit(0),
+    auto_hist(false),
     now_part(NULL),
     hist_part(NULL)
   {
     interval.type= INTERVAL_LAST;
   }
-  Vers_part_info(Vers_part_info &src) :
+  Vers_part_info(const Vers_part_info &src) :
     interval(src.interval),
     limit(src.limit),
+    auto_hist(src.auto_hist),
     now_part(NULL),
     hist_part(NULL)
   {
   }
+  Vers_part_info& operator= (const Vers_part_info &src)
+  {
+    interval= src.interval;
+    limit= src.limit;
+    auto_hist= src.auto_hist;
+    now_part= src.now_part;
+    hist_part= src.hist_part;
+    return *this;
+  }
   bool initialized()
   {
     if (now_part)
@@ -68,13 +74,19 @@ struct Vers_part_info : public Sql_alloc
     }
     return false;
   }
-  struct {
+  struct interval_t {
     my_time_t start;
     INTERVAL step;
     enum interval_type type;
-    bool is_set() { return type < INTERVAL_LAST; }
+    bool is_set() const { return type < INTERVAL_LAST; }
+    bool operator==(const interval_t &rhs) const
+    {
+      /* TODO: equivalent intervals like 1 hour and 60 mins should be considered equal */
+      return start == rhs.start && type == rhs.type && !memcmp(&step, &rhs.step, sizeof(INTERVAL));
+    }
   } interval;
   ulonglong limit;
+  bool auto_hist;
   partition_element *now_part;
   partition_element *hist_part;
 };
@@ -168,7 +180,7 @@ public:
     * lock_partitions  - partitions that must be locked (read or write).
     Usually read_partitions is the same set as lock_partitions, but
     in case of UPDATE the WHERE clause can limit the read_partitions set,
-    but not neccesarily the lock_partitions set.
+    but not necessarily the lock_partitions set.
     Usage pattern:
     * Initialized in ha_partition::open().
     * read+lock_partitions is set  according to explicit PARTITION,
@@ -267,7 +279,12 @@ public:
     {
       KEY_ALGORITHM_NONE= 0,
       KEY_ALGORITHM_51= 1,
-      KEY_ALGORITHM_55= 2
+      KEY_ALGORITHM_55= 2,
+      KEY_ALGORITHM_BASE31,
+      KEY_ALGORITHM_CRC32C,
+      KEY_ALGORITHM_XXH32,
+      KEY_ALGORITHM_XXH3,
+      KEY_ALGORITHM_END
     };
   enum_key_algorithm key_algorithm;
 
@@ -377,11 +394,13 @@ public:
   bool check_partition_field_length();
   bool init_column_part(THD *thd);
   bool add_column_list_value(THD *thd, Item *item);
-  partition_element *get_part_elem(const char *partition_name, char *file_name,
+  partition_element *get_part_elem(const Lex_ident_partition &partition_name,
+                                   char *file_name,
                                    size_t file_name_size, uint32 *part_id);
   void report_part_expr_error(bool use_subpart_expr);
   bool has_same_partitioning(partition_info *new_part_info);
   bool error_if_requires_values() const;
+  bool set_key_algorithm(const LEX_CSTRING *str);
 private:
   bool set_up_default_partitions(THD *thd, handler *file, HA_CREATE_INFO *info,
                                  uint start_no);
@@ -400,19 +419,14 @@ public:
   bool vers_init_info(THD *thd);
   bool vers_set_interval(THD *thd, Item *interval,
                          interval_type int_type, Item *starts,
-                         const char *table_name);
-  bool vers_set_limit(ulonglong limit)
-  {
-    DBUG_ASSERT(part_type == VERSIONING_PARTITION);
-    vers_info->limit= limit;
-    return !limit;
-  }
+                         bool auto_part, const char *table_name);
+  bool vers_set_limit(ulonglong limit, bool auto_part, const char *table_name);
+  bool vers_set_hist_part(THD* thd, uint *create_count);
   bool vers_require_hist_part(THD *thd) const
   {
     return part_type == VERSIONING_PARTITION &&
       thd->lex->vers_history_generating();
   }
-  int vers_set_hist_part(THD *thd);
   void vers_check_limit(THD *thd);
   bool vers_fix_field_list(THD *thd);
   void vers_update_el_ids();
@@ -437,6 +451,7 @@ void part_type_error(THD *thd, partition_info *work_part_info,
 
 uint32 get_next_partition_id_range(struct st_partition_iter* part_iter);
 bool check_partition_dirs(partition_info *part_info);
+bool vers_create_partitions(THD* thd, TABLE_LIST* tl, uint num_parts);
 
 /* Initialize the iterator to return a single partition with given part_id */
 
@@ -478,7 +493,13 @@ bool partition_info::vers_fix_field_list(THD * thd)
     return true;
   }
   DBUG_ASSERT(part_type == VERSIONING_PARTITION);
-  DBUG_ASSERT(table->versioned(VERS_TIMESTAMP));
+  if (!table->versioned(VERS_TIMESTAMP))
+  {
+    my_error(ER_VERS_FIELD_WRONG_TYPE, MYF(0),
+             table->vers_start_field()->field_name.str,
+             "TIMESTAMP(6)", table->s->table_name.str);
+    return true;
+  }
 
   Field *row_end= table->vers_end_field();
   // needed in handle_list_of_fields()
@@ -492,11 +513,6 @@ bool partition_info::vers_fix_field_list(THD * thd)
 }
 
 
-/**
-  @brief Update partition_element's id
-
-  @returns true on error; false on success
-*/
 inline
 void partition_info::vers_update_el_ids()
 {
@@ -519,11 +535,13 @@ void partition_info::vers_update_el_ids()
 }
 
 
-inline
-bool make_partition_name(char *move_ptr, uint i)
+static inline
+Lex_ident_partition make_partition_name(char *move_ptr, uint i)
 {
   int res= snprintf(move_ptr, MAX_PART_NAME_SIZE + 1, "p%u", i);
-  return res < 0 || res > MAX_PART_NAME_SIZE;
+  return res < 0 || res > MAX_PART_NAME_SIZE ?
+         Lex_ident_partition() :
+         Lex_ident_partition(move_ptr, (size_t) res);
 }
 
 
@@ -542,15 +560,16 @@ uint partition_info::next_part_no(uint new_parts) const
   for (uint cur_part= 0; cur_part < new_parts; ++cur_part, ++suffix)
   {
     uint32 cur_suffix= suffix;
-    if (make_partition_name(part_name, suffix))
+    Lex_ident_partition part_name_ls(make_partition_name(part_name, suffix));
+    if (!part_name_ls.str)
       return 0;
     partition_element *el;
     it.rewind();
     while ((el= it++))
     {
-      if (0 == my_strcasecmp(&my_charset_latin1, el->partition_name, part_name))
+      if (el->partition_name.streq(part_name_ls))
       {
-        if (make_partition_name(part_name, ++suffix))
+        if (!(part_name_ls= make_partition_name(part_name, ++suffix)).str)
           return 0;
         it.rewind();
       }
@@ -562,4 +581,27 @@ uint partition_info::next_part_no(uint new_parts) const
 }
 #endif
 
+
+class partition_element_iterator
+{
+  static List<partition_element> empty;
+  List_iterator_fast<partition_element> part_it, subpart_it;
+  public:
+  partition_element_iterator(List<partition_element> &partitions) :
+    part_it(partitions), subpart_it(empty) {}
+  partition_element *operator++(int)
+  {
+    partition_element *sub= subpart_it++;
+    if (sub)
+      return sub;
+    partition_element *part= part_it++;
+    if (!part)
+      return NULL;
+    subpart_it.init(part->subpartitions);
+    sub= subpart_it++;
+    return sub ? sub : part;
+  }
+};
+
+
 #endif /* PARTITION_INFO_INCLUDED */
