@@ -18,7 +18,7 @@
   @file
 
   Low level functions for storing data to be send to the MySQL client.
-  The actual communction is handled by the net_xxx functions in net_serv.cc
+  The actual communication is handled by the net_xxx functions in net_serv.cc
 */
 
 #include "mariadb.h"
@@ -64,7 +64,7 @@ bool Protocol_binary::net_store_data(const uchar *from, size_t length)
   net_store_data_cs() - extended version with character set conversion.
   
   It is optimized for short strings whose length after
-  conversion is garanteed to be less than 251, which accupies
+  conversion is guaranteed to be less than 251, which occupies
   exactly one byte to store length. It allows not to use
   the "convert" member as a temporary buffer, conversion
   is done directly to the "packet" member.
@@ -81,7 +81,7 @@ bool Protocol_binary::net_store_data_cs(const uchar *from, size_t length,
 #endif
 {
   uint dummy_errors;
-  /* Calculate maxumum possible result length */
+  /* Calculate maximum possible result length */
   size_t conv_length= to_cs->mbmaxlen * length / from_cs->mbminlen;
 
   if (conv_length > 250)
@@ -482,7 +482,7 @@ bool Protocol::net_send_error_packet(THD *thd, uint sql_errno, const char *err,
   We keep a separate version for that range because it's widely used in
   libmysql.
 
-  uint is used as agrument type because of MySQL type conventions:
+  uint is used as argument type because of MySQL type conventions:
   - uint for 0..65536
   - ulong for 0..4294967296
   - ulonglong for bigger numbers.
@@ -580,10 +580,9 @@ void Protocol::end_statement()
 #endif /* WITH_WSREP */
 
   DBUG_ENTER("Protocol::end_statement");
-  DBUG_ASSERT(! thd->get_stmt_da()->is_sent());
   bool error= FALSE;
 
-  /* Can not be true, but do not take chances in production. */
+  /* This can set in the case of early result sent by select_send::send_eof() */
   if (thd->get_stmt_da()->is_sent())
     DBUG_VOID_RETURN;
 
@@ -962,7 +961,7 @@ bool Protocol_text::store_field_metadata(const THD * thd,
   Detect whether column info can be changed without
   PS repreparing.
 
-  Such colum info is called fragile. The opposite of
+  Such column info is called fragile. The opposite of
   fragile is.
 
 
@@ -1696,6 +1695,7 @@ bool Protocol_text::send_out_parameters(List<Item_param> *sp_params)
 
     DBUG_ASSERT(sparam->get_item_param() == NULL);
     sparam->set_value(thd, thd->spcont, reinterpret_cast<Item **>(&item_param));
+    item_param->expr_event_handler(thd, expr_event_t::DESTRUCT_DYNAMIC_PARAM);
   }
 
   return FALSE;
@@ -1946,6 +1946,7 @@ bool Protocol_binary::send_out_parameters(List<Item_param> *sp_params)
       if (!item_param->get_out_param_info())
         continue; // It's an IN-parameter.
 
+      item_param->expr_event_handler(thd, expr_event_t::DESTRUCT_DYNAMIC_PARAM);
       if (out_param_lst.push_back(item_param, thd->mem_root))
         return TRUE;
     }
