@@ -113,15 +113,15 @@ pack_row(TABLE *table, MY_BITMAP const* cols,
           length is stored in little-endian format, since this is the
           format used for the binlog.
         */
-#ifndef DBUG_OFF
+#if !defined DBUG_OFF && defined DBUG_TRACE
         const uchar *old_pack_ptr= pack_ptr;
 #endif
         pack_ptr= field->pack(pack_ptr, field->ptr + offset);
-        DBUG_PRINT("debug", ("field: %s; real_type: %d, pack_ptr: %p;"
-                             " pack_ptr':%p; bytes: %d",
-                             field->field_name.str, field->real_type(),
-                             old_pack_ptr,pack_ptr,
-                             (int) (pack_ptr - old_pack_ptr)));
+        DBUG_PRINT("rpl_record",
+                   ("field: %s; real_type: %d, pack_ptr: %p;"
+                    " pack_ptr':%p; bytes: %d",
+                    field->field_name.str, field->real_type(), old_pack_ptr,
+                    pack_ptr, (int) (pack_ptr - old_pack_ptr)));
         DBUG_DUMP("packed_data", old_pack_ptr, pack_ptr - old_pack_ptr);
       }
 
@@ -175,98 +175,64 @@ struct Unpack_record_state
     null_ptr(row_data), pack_ptr(row_data + master_null_byte_count),
     null_pos(0)
   {}
+
+  /*
+    When unpacking a row, if no value was provided for a field (i.e. it is
+    NULL), the field needs to be prepared because there may be an existing
+    record in the table with data. Nullable fields are reset, and non-null
+    fields are set to their default value.
+  */
+  void prepare_null_field(Field *f)
+  {
+    /*
+      Ensure that the null bit for the current field is set. One is subtracted
+      from the null_pos because the null_pos is incremented before calling into
+      this function.
+    */
+    DBUG_ASSERT(this->null_pos);
+    DBUG_ASSERT(rpl_bitmap_is_set(this->null_ptr, this->null_pos - 1));
+
+    if (f->maybe_null())
+    {
+      DBUG_FIX_WRITE_SET(f, f->table->write_set);
+      f->reset();
+      DBUG_RESTORE_WRITE_SET(f, f->table->write_set);
+      f->set_null();
+    }
+    else
+    {
+      THD *thd= f->table->in_use;
+
+      f->set_default();
+      push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
+                          ER_BAD_NULL_ERROR, ER_THD(thd, ER_BAD_NULL_ERROR),
+                          f->field_name.str);
+    }
+  };
+
+  /*
+    Unpack the value from a packed row into a field. Field must be non-null.
+
+    Returns true if the field was unpacked successfully, false otherwise.
+  */
+  bool unpack_field(const table_def *tabledef, Field *f, uint master_field_idx)
+  {
+    DBUG_ASSERT(!f->is_null());
+    uint16 const metadata= tabledef->field_metadata(master_field_idx);
+#ifdef DBUG_TRACE
+    uchar const *const old_pack_ptr= this->pack_ptr;
+#endif
+    this->pack_ptr= f->unpack(f->ptr, this->pack_ptr, this->row_end, metadata);
+    DBUG_PRINT("rpl_record",
+               ("field: %s; metadata: 0x%x;"
+                " pack_ptr: %p; pack_ptr': %p; bytes: %d",
+                f->field_name.str, metadata, old_pack_ptr, this->pack_ptr,
+                (int) (this->pack_ptr - old_pack_ptr)));
+
+    return MY_TEST(this->pack_ptr);
+  };
 };
 
-/*
-  When unpacking a row, if no value was provided for a field (i.e. it is NULL),
-  the field needs to be prepared because there may be an existing record in the
-  table with data. Nullable fields are reset, and non-null fields are set to
-  their default value.
-*/
-static void prepare_null_field(Field *f, Unpack_record_state *st)
-{
-  /*
-    Ensure that the null bit for the current field is set. One is subtracted
-    from the null_pos because the null_pos is incremented before calling into
-    this function.
-  */
-  DBUG_ASSERT(st->null_pos);
-  DBUG_ASSERT(rpl_bitmap_is_set(st->null_ptr, st->null_pos - 1));
-
-  if (f->maybe_null())
-  {
-    /**
-      Calling reset just in case one is unpacking on top a
-      record with data.
-
-      This could probably go into set_null() but doing so,
-      (i) triggers assertion in other parts of the code at
-      the moment; (ii) it would make us reset the field,
-      always when setting null, which right now doesn't seem
-      needed anywhere else except here.
-
-      TODO: maybe in the future we should consider moving
-            the reset to make it part of set_null. But then
-            the assertions triggered need to be
-            addressed/revisited.
-     */
-
-#ifndef DBUG_OFF
-    /*
-      f->reset() may call store_value() to reset the value, for example
-      Field_new_decimal. store_value() has below assertion:
-
-      DBUG_ASSERT(marked_for_write_or_computed());
-
-      It asserts write bitmap must be set. That caused an assertion
-      failure for row images generated by FULL_NODUP mode.
-      The assertion is meaningless for unpacking a row image, so
-      the field is marked in write_set temporarily to avoid the
-      assertion failure.
-    */
-    bool was_not_set= !bitmap_is_set(f->table->write_set, f->field_index);
-    if (was_not_set)
-      bitmap_set_bit(f->table->write_set, f->field_index);
-#endif
-    f->reset();
-#ifndef DBUG_OFF
-    if (was_not_set)
-      bitmap_clear_bit(f->table->write_set, f->field_index);
-#endif
-    f->set_null();
-  }
-  else
-  {
-    THD *thd= f->table->in_use;
-
-    f->set_default();
-    push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN, ER_BAD_NULL_ERROR,
-                        ER_THD(thd, ER_BAD_NULL_ERROR), f->field_name.str);
-  }
-}
-
-/*
-  Unpack the value from a packed row into a field. Field must be non-null.
-
-  Returns true if the field was unpacked successfully, false otherwise.
-*/
-static bool unpack_field(const table_def *tabledef, Field *f,
-                         Unpack_record_state *st, uint field_idx)
-{
-  DBUG_ASSERT(!f->is_null());
-  uint16 const metadata= tabledef->field_metadata(field_idx);
-#ifdef DBUG_TRACE
-  uchar const *const old_pack_ptr= st->pack_ptr;
-#endif
-
-  st->pack_ptr= f->unpack(f->ptr, st->pack_ptr, st->row_end, metadata);
-  DBUG_PRINT("debug", ("field: %s; metadata: 0x%x;"
-                       " pack_ptr: %p; pack_ptr': %p; bytes: %d",
-                       f->field_name.str, metadata, old_pack_ptr, st->pack_ptr,
-                       (int) (st->pack_ptr - old_pack_ptr)));
-
-  return static_cast<bool>(st->pack_ptr);
-}
 
 /*
   While unpacking a row, if the type of the field on the master is different
@@ -286,9 +252,10 @@ static void convert_field(Field *result_field, Field *conv_field)
   String value_string(value_buf, sizeof(value_buf), system_charset_info);
   conv_field->sql_type(source_type);
   conv_field->val_str(&value_string);
-  DBUG_PRINT("debug", ("Copying field '%s' of type '%s' with value '%s'",
-                       result_field->field_name.str,
-                       source_type.c_ptr_safe(), value_string.c_ptr_safe()));
+  DBUG_PRINT("rpl_record",
+             ("Copying field '%s' of type '%s' with value '%s'",
+              result_field->field_name.str, source_type.c_ptr_safe(),
+              value_string.c_ptr_safe()));
 #endif
 
   Copy_field copy;
@@ -299,51 +266,13 @@ static void convert_field(Field *result_field, Field *conv_field)
   String target_type(type_buf, sizeof(type_buf), system_charset_info);
   result_field->sql_type(target_type);
   result_field->val_str(&value_string);
-  DBUG_PRINT("debug", ("Value of field '%s' of type '%s' is now '%s'",
-                       result_field->field_name.str,
-                       target_type.c_ptr_safe(), value_string.c_ptr_safe()));
+  DBUG_PRINT("rpl_record",
+             ("Value of field '%s' of type '%s' is now '%s'",
+              result_field->field_name.str, target_type.c_ptr_safe(),
+              value_string.c_ptr_safe()));
 #endif
 }
 
-/**
-  Updates a table's write_set to include slave-only fields that are
-  automatically filled in (either with a default or virtual column value). That
-  is, when replicating a rows log event, a table's write_set is initially
-  determined by the event's column bitmaps (in the case of an update rows
-  event, it is the after_image bitmap). However, if a field isn't present on
-  the master, the binlog event's column mapping won't be able to include it; so
-  we iterate through a table's fields which will be automatically populated,
-  and add them to the write_set.
-
-  @param table           Table to update the write_set for
-  @param field_start_ptr Pointer to the first automatically populatable field
-                         of the table (e.g. table->default_field or
-                         table->vfield).
-*/
-static void update_write_set_for_auto_filled_fields(TABLE *table,
-                                                    Field **field_start_ptr)
-{
-  DBUG_ENTER("update_write_set_for_auto_filled_fields");
-  if (!field_start_ptr || !*field_start_ptr)
-    DBUG_VOID_RETURN;
-
-  Field **field_ptr, *field;
-  for (field_ptr= field_start_ptr; *field_ptr; ++field_ptr)
-  {
-    field= *field_ptr;
-    /*
-      We only want to automatically populate the value of fields which don't
-      have values provided by the master; so we check that either no value was
-      provided, or the table's original write set accounts for the explicit
-      value.
-    */
-    DBUG_ASSERT(!field->has_explicit_value() ||
-                bitmap_is_set(table->write_set, field->field_index));
-    if (field->stored_in_db())
-      bitmap_set_bit(table->write_set, field->field_index);
-  }
-  DBUG_VOID_RETURN;
-}
 
 /**
    Unpack a row into @c table->record[0].
@@ -363,7 +292,7 @@ static void update_write_set_for_auto_filled_fields(TABLE *table,
    @note The relay log information can be NULL, which means that no
    checking or comparison with the source table is done, simply
    because it is not used.  This feature is used by MySQL Backup to
-   unpack a row from from the backup image, but can be used for other
+   unpack a row from the backup image, but can be used for other
    purposes as well.
 
    @param rgi     Relay group info
@@ -375,9 +304,6 @@ static void update_write_set_for_auto_filled_fields(TABLE *table,
    @param curr_row_end
                   Pointer to variable that will hold the value of the
                   one-after-end position for the current row
-   @param master_reclength
-                  Pointer to variable that will be set to the length of the
-                  record on the master side
    @param row_end
                   Pointer to variable that will hold the value of the
                   end position for the data in the row event
@@ -393,7 +319,7 @@ static void update_write_set_for_auto_filled_fields(TABLE *table,
 int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
                uchar const *const row_data, MY_BITMAP const *cols,
                uchar const **const current_row_end,
-               ulong *const master_reclength, uchar const *const row_end)
+               uchar const *const row_end)
 {
   int error;
   bool null_value;
@@ -411,21 +337,21 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
        nothing to unpack.
      */
     *current_row_end= st.pack_ptr;
-    *master_reclength= 0;
     DBUG_RETURN(0);
   }
 
   Rpl_table_data rpl_data= *(RPL_TABLE_LIST*)table->pos_in_table_list;
   const table_def *tabledef= rpl_data.tabledef;
   const TABLE *conv_table= rpl_data.conv_table;
-  uint conv_table_idx= 0, master_idx= 0;
-  DBUG_PRINT("debug", ("Table data: tabldef: %p, conv_table: %p",
-                       tabledef, conv_table));
+  uint conv_table_idx= 0;
+  DBUG_PRINT("rpl_record", ("Table data: tabldef: %p, conv_table: %p",
+                            tabledef, conv_table));
 
   /*
-    A slave needs additional checks when unpacking a row than from the ONLINE
-    ALTER use case. The slave must account for its tables having either columns
-    in different positions,  or with different types, than on the master.
+    When unpacking a row for replication (rather than online alter), it needs
+    its own additional checks. The slave must account for its tables having
+    either columns in different positions, or with different types, than on the
+    master.
   */
   if (!rpl_data.is_online_alter())
   {
@@ -442,11 +368,10 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
         2. Unpack the actual value into the slave field with any necessary
            conversions.
     */
-    for (; master_idx < master_cols; master_idx++)
+    for (uint master_idx= 0; master_idx < master_cols; master_idx++)
     {
       Field *field=NULL, *conv_field= NULL;
       /*
-
         Check 1: Skip unpacking if the field wasn't written in this record.
         This can happen for update row events when the before_image and
         after_image are disjoint (e.g. when binlogged with
@@ -454,7 +379,6 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
       */
       if (!bitmap_is_set(cols, master_idx))
       {
-
         /*
           Check if we need to update the conv_table_idx. A field is only added
           to the conv_table when it exists on the slave.
@@ -463,21 +387,7 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
           table, but
         */
         if (!(tabledef->master_to_slave_error[master_idx])) // field exists on slave
-        {
-          /*
-            Review-only-comment: Removed this assertion becaues it assumes that
-            the table->write_set is a superset of the read and write sets, but
-            it isn't. When an update row event is binlogged with
-            binlog_row_image=MINIMAL, the read and write sets can be disjoint.
-            As this is is only used for debug assertions, I decided to remove
-            the assertion altogether, rather than have a mode of callling the
-            function in to use either the read or write set.
-          */
-          //DBUG_ASSERT(!bitmap_is_set(table->write_set,
-          //                           master_to_slave_map[master_idx]));
           conv_table_idx++;
-        }
-
         continue;
       }
 
@@ -497,11 +407,11 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
         continue;
       }
 
-
       uint slave_idx= tabledef->master_to_slave_map[master_idx];
       DBUG_ASSERT(bitmap_is_set(table->write_set, slave_idx) ||
                   bitmap_is_set(table->read_set, slave_idx));
       result_field= field= table->field[slave_idx];
+      result_field->set_has_explicit_value();
 
       /*
         Check 3: Skip unpacking if NULL is explicitly provided for the field.
@@ -510,21 +420,16 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
       */
       if (null_value)
       {
-        prepare_null_field(field, &st);
+        st.prepare_null_field(field);
         conv_table_idx++;
         continue;
       }
+      result_field->set_notnull();
 
       /*
         Phase 2: Unpack the actual value into the slave table with any
         necessary conversions.
-      */
 
-      /* Set attributes for the slave-side field */
-      result_field->set_has_explicit_value();
-      result_field->set_notnull();
-
-      /*
         If there is a conversion table, we pick up the field pointer to
         the conversion table.  If the conversion table or the field
         pointer is NULL, no conversions are necessary.
@@ -532,13 +437,12 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
       if (conv_table && (conv_field= conv_table->field[conv_table_idx++]))
         field= conv_field;
 
-
-      DBUG_PRINT("debug", ("Conversion %srequired for field '%s' (#%u)",
-                           conv_field ? "" : "not ",
-                           result_field->field_name.str, master_idx));
+      DBUG_PRINT("rpl_record", ("Conversion %srequired for field '%s' (#%u)",
+                                conv_field ? "" : "not ",
+                                result_field->field_name.str, master_idx));
       DBUG_ASSERT(field != NULL);
 
-      bool unpack_result= unpack_field(tabledef, field, &st, master_idx);
+      bool unpack_result= st.unpack_field(tabledef, field, master_idx);
       if (!unpack_result)
       {
         rgi->rli->report(ERROR_LEVEL, ER_SLAVE_CORRUPT_EVENT,
@@ -561,14 +465,6 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
       if (conv_field)
         convert_field(result_field, conv_field);
     }
-
-    if (master_reclength)
-    {
-      if (result_field)
-        *master_reclength = (ulong)(result_field->ptr - table->record[0]);
-      else
-        *master_reclength = table->s->reclength;
-    }
   }
   else
   {
@@ -578,32 +474,28 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
      */
 
     DBUG_ASSERT(master_cols == conv_table->s->fields);
-    for (;master_idx < master_cols; master_idx++)
+    for (uint master_idx= 0; master_idx < master_cols; master_idx++)
     {
       DBUG_ASSERT(bitmap_is_set(cols, master_idx));
       Field *f= conv_table->field[master_idx];
       if (rpl_bitmap_is_set(st.null_ptr, st.null_pos++))
       {
-        prepare_null_field(f, &st);
+        st.prepare_null_field(f);
         continue;
       }
       f->set_notnull();
       f->set_has_explicit_value();
-#ifndef DBUG_OFF
-      bool result=
-#endif
-        unpack_field(tabledef, f, &st, master_idx);
+      bool result __attribute__((unused))=
+          st.unpack_field(tabledef, f, master_idx);
       DBUG_ASSERT(result);
     }
 
-    for (const auto *copy=rpl_data.copy_fields;
+    for (const Copy_field *copy=rpl_data.copy_fields;
          copy != rpl_data.copy_fields_end; copy++)
     {
       copy->to_field->set_has_explicit_value();
       copy->do_copy(copy);
     }
-    if (master_reclength)
-      *master_reclength = conv_table->s->reclength;
   } // if (rpl_data.is_online_alter())
 
   /*
@@ -615,10 +507,13 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
 
   *current_row_end = st.pack_ptr;
 
-  if (table->default_field && (rpl_data.is_online_alter() ||
-      LOG_EVENT_IS_WRITE_ROW(rgi->current_event->get_type_code())))
+  DBUG_ASSERT(!table->default_field || table->default_field[0]);
+  DBUG_ASSERT(!table->vfield || table->vfield[0]);
+
+  if (table->default_field &&
+      (rpl_data.is_online_alter() ||
+       LOG_EVENT_IS_WRITE_ROW(rgi->current_event->get_type_code())))
   {
-    update_write_set_for_auto_filled_fields(table, table->default_field);
     error= table->update_default_fields(table->in_use->lex->ignore);
     if (unlikely(error))
       DBUG_RETURN(error);
@@ -632,7 +527,6 @@ int unpack_row(const rpl_group_info *rgi, TABLE *table, uint const master_cols,
       If a master provides values for a persisted virtual column, the slave
       overwrites these values using its own function.
     */
-    update_write_set_for_auto_filled_fields(table, table->vfield);
     error= table->update_virtual_fields(table->file, VCOL_UPDATE_FOR_WRITE);
     if (unlikely(error))
       DBUG_RETURN(error);
