@@ -32,11 +32,11 @@ size_t pack_row(TABLE* table, MY_BITMAP const* cols,
 int unpack_row(const rpl_group_info *rgi,
                TABLE *table, uint const colcnt,
                uchar const *const row_data, MY_BITMAP const *cols,
-               uchar const **const curr_row_end, ulong *const master_reclength,
+               uchar const **const curr_row_end,
                uchar const *const row_end);
 
 // Fill table's record[0] with default values.
-int prepare_record(TABLE *const table, const uint skip, const bool check);
+int prepare_record(TABLE *const table);
 #endif
 
 #endif
