@@ -1036,16 +1036,13 @@ bool RPL_TABLE_LIST::check_wrong_column_usage(rpl_group_info *rgi,
     if (!bitmap_is_set(m_cols, col))
       continue;
     if (m_tabledef.master_to_slave_error[col])
-    {
-      has_err= give_compatibility_error(rgi, col) || has_err;
-    }
-    DBUG_ASSERT(m_tabledef.master_column_name[col] == NULL);
+      has_err|= give_compatibility_error(rgi, col);
   }
   DBUG_RETURN(has_err);
 }
 
 /*
-  Give an error if we are trying to access a wrong column
+  Report errors for any (and all) incompatible columns that we try to access.
 
   @return 0  error was ignored
   @return 1  error, abort replication
@@ -1059,7 +1056,6 @@ bool RPL_TABLE_LIST::give_compatibility_error(rpl_group_info *rgi, uint col)
   switch (m_tabledef.master_to_slave_error[col]) {
   case SLAVE_FIELD_NAME_MISSING:
     DBUG_ASSERT(m_tabledef.master_column_name[col]);
-    DBUG_ASSERT(m_tabledef.master_to_slave_map[col] == UINT_MAX32);
     if (!(slave_type_conversions_options &
           (1ULL << SLAVE_TYPE_CONVERSIONS_ERROR_IF_MISSING_FIELD)))
       error_level= WARNING_LEVEL;
@@ -1073,8 +1069,6 @@ bool RPL_TABLE_LIST::give_compatibility_error(rpl_group_info *rgi, uint col)
             error_level, ER_SLAVE_INCOMPATIBLE_TABLE_DEF, rgi->gtid_info(),
             ER_THD(rgi->thd, ER_SLAVE_INCOMPATIBLE_TABLE_DEF), error_msg);
       }
-    my_free(m_tabledef.master_column_name[col]);
-    m_tabledef.master_column_name[col]= NULL;
     break;
   case SLAVE_FIELD_NR_MISSING:
   {
@@ -1322,14 +1316,13 @@ void Deferred_log_events::rewind()
 
   Mapping stored in master_to_slave_map[].
   Errors stored in master_to_slave_error[]. Error will be given
-  on usage.
+  on usage (see ::check_wrong_column_usage()).
   Store master column names in master_column_name[].
 
   Note that we map all columns as we at this point do not know which
   columns will be used by the row events.
 */
-
-bool RPL_TABLE_LIST::create_column_mapping(rpl_group_info *rgi)
+void RPL_TABLE_LIST::create_column_mapping(rpl_group_info *rgi)
 {
   ulong master_cols= m_tabledef.size();
   DBUG_ENTER("RPL_TABLE_LIST::create_column_mapping");
@@ -1351,14 +1344,17 @@ default_column_mapping:
       m_tabledef.master_to_slave_map[col]= UINT_MAX32;
       m_tabledef.master_to_slave_error[col]= SLAVE_FIELD_NR_MISSING;
     }
-    DBUG_RETURN(0);
+    DBUG_VOID_RETURN;
   }
 
-  Table_map_log_event::Optional_metadata_fields
-    opt_metadata((uchar*) m_tabledef.optional_metadata.str,
-                 m_tabledef.optional_metadata.length);
+  /* Note that the following may fail with out of memory. Need to be handled */
 
-  if (!opt_metadata.m_column_name.size())
+  Table_map_log_event::Optional_metadata_fields
+    opt_metadata(rgi->thd->mem_root, master_cols,
+                 (uchar*) m_tabledef.optional_metadata.str,
+                 m_tabledef.optional_metadata.length, 1);
+
+  if (!opt_metadata.m_column_name)
   {
     /*
       If there are no column names provided in the optional metadata
@@ -1370,27 +1366,11 @@ default_column_mapping:
 
   for (uint col= 0; col < master_cols; col++)
   {
-    std::string master_col_name_cppstr= opt_metadata.m_column_name[col];
-    LEX_CSTRING field_name=
-      { master_col_name_cppstr.c_str(), master_col_name_cppstr.length() };
-    Field *field= table->find_field_by_name(&field_name);
+    const LEX_CSTRING *field_name= &opt_metadata.m_column_name[col];
+    Field *field= table->find_field_by_name(field_name);
     if (unlikely(!field))
     {
-      DBUG_ASSERT(m_tabledef.master_column_name[col] == NULL);
-
-      /*
-        This field name will be referenced later in the execution path when
-        writing errors/warnings, so allocate memory to hold the table name, as
-        the ones that currently exist (opt_metadata.m_column_name[col] and
-        field_name) are stored on the stack.
-      */
-      size_t field_name_sz= master_col_name_cppstr.size();
-      m_tabledef.master_column_name[col]= (char *) my_malloc(
-          PSI_INSTRUMENT_ME, field_name_sz * sizeof(char) + 1, MYF(MY_WME));
-      strncpy(m_tabledef.master_column_name[col],
-              master_col_name_cppstr.c_str(), field_name_sz);
-      m_tabledef.master_column_name[col][field_name_sz] = '\0';
-
+      m_tabledef.master_column_name[col]= field_name->str;
       /*
         Note master_to_slave_map[col] is set to UINT_MAX32, but is never
         actually used - the master_to_slave_error check always happens
@@ -1398,12 +1378,12 @@ default_column_mapping:
       */
       m_tabledef.master_to_slave_map[col]= UINT_MAX32;
       m_tabledef.master_to_slave_error[col]= SLAVE_FIELD_NAME_MISSING;
-      continue;                               // ok that field did not exists
+      continue;                               // ok that field did not exist
     }
     m_tabledef.master_to_slave_map[col]= field->field_index;
-    DBUG_PRINT("info", ("Found mapping for %s", field_name.str));
+    DBUG_PRINT("info", ("Found mapping for %s", field_name->str));
   }
-  DBUG_RETURN(false);
+  DBUG_VOID_RETURN;
 }
 
 #endif // defined(HAVE_REPLICATION)
