@@ -167,7 +167,7 @@ protected:
   /** Severity (error, warning, note) of this condition. */
   enum_warning_level m_level;
 
-  void assign_defaults(const Sql_state_errno *value);
+  void assign_defaults(THD *thd, const Sql_state_errno *value);
 
 public:
   /**
@@ -201,7 +201,7 @@ public:
 
 /*
   class Sql_user_condition_identity.
-  Instances of this class uniquely idetify user defined conditions (EXCEPTION).
+  Instances of this class uniquely identify user defined conditions (EXCEPTION).
 
     SET sql_mode=ORACLE;
     CREATE PROCEDURE p1
@@ -403,7 +403,7 @@ private:
 
   /**
     Default constructor.
-    This constructor is usefull when allocating arrays.
+    This constructor is useful when allocating arrays.
     Note that the init() method should be called to complete the Sql_condition.
   */
   Sql_condition()
@@ -1097,6 +1097,11 @@ public:
     return m_affected_rows;
   }
 
+  void set_message(const char *msg)
+  {
+    strmake_buf(m_message, msg);
+  }
+
   ulonglong last_insert_id() const
   {
     DBUG_ASSERT(m_status == DA_OK || m_status == DA_OK_BULK);
@@ -1327,7 +1332,11 @@ void push_warning(THD *thd, Sql_condition::enum_warning_level level,
                   uint code, const char *msg);
 
 void push_warning_printf(THD *thd, Sql_condition::enum_warning_level level,
-                         uint code, const char *format, ...);
+                         uint code, const char *format, ...)
+                         ATTRIBUTE_FORMAT(printf, 4, 5);
+
+void push_warning_vprintf(THD *thd, Sql_condition::enum_warning_level level,
+                          uint code, const char *format, va_list args);
 
 bool mysqld_show_warnings(THD *thd, ulong levels_to_show);
 
