@@ -660,6 +660,20 @@ bool trans_savepoint(THD *thd, LEX_CSTRING name)
   if (thd->transaction->xid_state.check_has_uncommitted_xa())
     DBUG_RETURN(TRUE);
 
+  if (unlikely(thd->in_sub_stmt & SUB_STMT_TRIGGER) &&
+      opt_binlog_engine_hton)
+  {
+    /*
+      ToDo: Probably using savepoints in triggers should be disallowed
+      unconditionally. For now, we disallow it in the new binlog
+      implementation, which is not enabled by default, so that backwards
+      compatibility is of smaller concern.
+    */
+    my_error(ER_NOT_AVAILABLE_WITH_ENGINE_BINLOG, MYF(0),
+             "Using savepoints in triggers");
+    DBUG_RETURN(1);
+  }
+
   SAVEPOINT *newsv= savepoint_add(thd, Lex_ident_savepoint(name),
                                   &thd->transaction->savepoints,
                                   ha_release_savepoint);
@@ -713,10 +727,24 @@ bool trans_savepoint(THD *thd, LEX_CSTRING name)
 bool trans_rollback_to_savepoint(THD *thd, LEX_CSTRING name)
 {
   int res= FALSE;
-  SAVEPOINT *sv= *find_savepoint(thd, Lex_ident_savepoint(name));
+  SAVEPOINT *sv;
   DBUG_ENTER("trans_rollback_to_savepoint");
 
-  if (sv == NULL)
+  if (unlikely(thd->in_sub_stmt & SUB_STMT_TRIGGER) &&
+      opt_binlog_engine_hton)
+  {
+    /*
+      ToDo: Probably using savepoints in triggers should be disallowed
+      unconditionally. For now, we disallow it in the new binlog
+      implementation, which is not enabled by default, so that backwards
+      compatibility is of smaller concern.
+    */
+    my_error(ER_NOT_AVAILABLE_WITH_ENGINE_BINLOG, MYF(0),
+             "Using savepoints in triggers");
+    DBUG_RETURN(1);
+  }
+
+  if ((sv= *find_savepoint(thd, Lex_ident_savepoint(name))) == NULL)
   {
     my_error(ER_SP_DOES_NOT_EXIST, MYF(0), "SAVEPOINT", name.str);
     DBUG_RETURN(TRUE);
