@@ -468,88 +468,6 @@ fts_ast_state_free(
 	state->root = state->list.head = state->list.tail = NULL;
 }
 
-/** Print the ast string
-@param[in] str		string to print */
-static
-void
-fts_ast_string_print(
-	const fts_ast_string_t*	ast_str)
-{
-	for (ulint i = 0; i < ast_str->len; ++i) {
-		printf("%c", ast_str->str[i]);
-	}
-
-	printf("\n");
-}
-
-/******************************************************************//**
-Print an ast node recursively. */
-static
-void
-fts_ast_node_print_recursive(
-/*=========================*/
-	fts_ast_node_t*	node,			/*!< in: ast node to print */
-	ulint		level)			/*!< in: recursive level */
-{
-	/* Print alignment blank */
-	for (ulint i = 0; i < level; i++) {
-		printf("  ");
-	}
-
-	switch (node->type) {
-	case FTS_AST_TEXT:
-		printf("TEXT: ");
-		fts_ast_string_print(node->text.ptr);
-		break;
-
-	case FTS_AST_TERM:
-		printf("TERM: ");
-		fts_ast_string_print(node->term.ptr);
-		break;
-
-	case FTS_AST_LIST:
-		printf("LIST: \n");
-
-		for (node = node->list.head; node; node = node->next) {
-			fts_ast_node_print_recursive(node, level + 1);
-		}
-		break;
-
-	case FTS_AST_SUBEXP_LIST:
-		printf("SUBEXP_LIST: \n");
-
-		for (node = node->list.head; node; node = node->next) {
-			fts_ast_node_print_recursive(node, level + 1);
-		}
-		break;
-
-	case FTS_AST_OPER:
-		printf("OPER: %d\n", node->oper);
-		break;
-
-	case FTS_AST_PARSER_PHRASE_LIST:
-		printf("PARSER_PHRASE_LIST: \n");
-
-		for (node = node->list.head; node; node = node->next) {
-			fts_ast_node_print_recursive(node, level + 1);
-		}
-		break;
-
-	default:
-		ut_error;
-	}
-}
-
-/******************************************************************//**
-Print an ast node */
-void
-fts_ast_node_print(
-/*===============*/
-	fts_ast_node_t* node)		/*!< in: ast node to print */
-{
-	fts_ast_node_print_recursive(node, 0);
-}
-
 /** Check only union operation involved in the node
 @param[in]	node	ast node to check
 @return true if the node contains only union else false. */
@@ -688,7 +606,7 @@ fts_ast_visit(
 				continue;
 			}
 
-			/* Process leaf node accroding to its pass.*/
+			/* Process leaf node according to its pass.*/
 			if (oper == FTS_EXIST_SKIP
 			    && visit_pass == FTS_PASS_EXIST) {
 				error = visitor(FTS_EXIST, node, arg);
