@@ -119,11 +119,7 @@ fts_config_get_value(
 	trx->op_info = "getting FTS config value";
 
 	error = fts_eval_sql(trx, graph);
-
-	mutex_enter(&dict_sys.mutex);
 	que_graph_free(graph);
-	mutex_exit(&dict_sys.mutex);
-
 	return(error);
 }
 
@@ -230,12 +226,12 @@ fts_config_set_value(
 
 	error = fts_eval_sql(trx, graph);
 
-	fts_que_graph_free_check_lock(fts_table, NULL, graph);
+	que_graph_free(graph);
 
 	n_rows_updated = trx->undo_no - undo_no;
 
 	/* Check if we need to do an insert. */
-	if (n_rows_updated == 0) {
+	if (error == DB_SUCCESS && n_rows_updated == 0) {
 		info = pars_info_create();
 
 		pars_info_bind_varchar_literal(
@@ -256,7 +252,7 @@ fts_config_set_value(
 
 		error = fts_eval_sql(trx, graph);
 
-		fts_que_graph_free_check_lock(fts_table, NULL, graph);
+		que_graph_free(graph);
 	}
 
 	return(error);
@@ -292,76 +288,6 @@ fts_config_set_index_value(
 	return(error);
 }
 
-#ifdef FTS_OPTIMIZE_DEBUG
-/******************************************************************//**
-Get an ulint value from the config table.
-@return DB_SUCCESS if all OK else error code */
-dberr_t
-fts_config_get_index_ulint(
-/*=======================*/
-	trx_t*		trx,			/*!< in: transaction */
-	dict_index_t*	index,			/*!< in: FTS index */
-	const char*	name,			/*!< in: param name */
-	ulint*		int_value)		/*!< out: value */
-{
-	dberr_t		error;
-	fts_string_t	value;
-
-	/* We set the length of value to the max bytes it can hold. This
-	information is used by the callback that reads the value.*/
-	value.f_len = FTS_MAX_CONFIG_VALUE_LEN;
-	value.f_str = static_cast<byte*>(ut_malloc_nokey(value.f_len + 1));
-
-	error = fts_config_get_index_value(trx, index, name, &value);
-
-	if (UNIV_UNLIKELY(error != DB_SUCCESS)) {
-		ib::error() << "(" << error << ") reading `" << name << "'";
-	} else {
-		*int_value = strtoul((char*) value.f_str, NULL, 10);
-	}
-
-	ut_free(value.f_str);
-
-	return(error);
-}
-
-/******************************************************************//**
-Set an ulint value in the config table.
-@return DB_SUCCESS if all OK else error code */
-dberr_t
-fts_config_set_index_ulint(
-/*=======================*/
-	trx_t*		trx,			/*!< in: transaction */
-	dict_index_t*	index,			/*!< in: FTS index */
-	const char*	name,			/*!< in: param name */
-	ulint		int_value)		/*!< in: value */
-{
-	dberr_t		error;
-	fts_string_t	value;
-
-	/* We set the length of value to the max bytes it can hold. This
-	information is used by the callback that reads the value.*/
-	value.f_len = FTS_MAX_CONFIG_VALUE_LEN;
-	value.f_str = static_cast<byte*>(ut_malloc_nokey(value.f_len + 1));
-
-	// FIXME: Get rid of snprintf
-	ut_a(FTS_MAX_INT_LEN < FTS_MAX_CONFIG_VALUE_LEN);
-
-	value.f_len = snprintf(
-		(char*) value.f_str, FTS_MAX_INT_LEN, ULINTPF, int_value);
-
-	error = fts_config_set_index_value(trx, index, name, &value);
-
-	if (UNIV_UNLIKELY(error != DB_SUCCESS)) {
-		ib::error() << "(" << error << ") writing `" << name << "'";
-	}
-
-	ut_free(value.f_str);
-
-	return(error);
-}
-#endif /* FTS_OPTIMIZE_DEBUG */
-
 /******************************************************************//**
 Get an ulint value from the config table.
 @return DB_SUCCESS if all OK else error code */
