@@ -88,7 +88,7 @@ those defined in mysql file ft_global.h */
 
 #define FTS_INDEX_TABLE_IND_NAME	"FTS_INDEX_TABLE_IND"
 
-/** The number of FTS index partitions for a fulltext idnex */
+/** The number of FTS index partitions for a fulltext index */
 #define FTS_NUM_AUX_INDEX		6
 
 /** Threshold where our optimize thread automatically kicks in */
@@ -125,10 +125,6 @@ extern ulong		fts_sort_pll_degree;
 call */
 extern ulong		fts_num_word_optimize;
 
-/** Variable specifying whether we do additional FTS diagnostic printout
-in the log */
-extern char		fts_enable_diag_print;
-
 /** FTS rank type, which will be between 0 .. 1 inclusive */
 typedef float 		fts_rank_t;
 
