@@ -46,7 +46,6 @@ typedef struct mem_block_info_t mem_heap_t;
 // JAN: TODO missing features:
 #undef MYSQL_FT_INIT_EXT
 #undef MYSQL_PFS
-#undef MYSQL_STORE_FTS_DOC_ID
 
 /*******************************************************************//**
 Formats the raw data in "data" (in InnoDB on-disk format) that is of
@@ -329,16 +328,6 @@ innobase_next_autoinc(
 	ulonglong	max_value)	/*!< in: max value for type */
 	MY_ATTRIBUTE((pure, warn_unused_result));
 
-/**********************************************************************
-Check if the length of the identifier exceeds the maximum allowed.
-The input to this function is an identifier in charset my_charset_filename.
-return true when length of identifier is too long. */
-my_bool
-innobase_check_identifier_length(
-/*=============================*/
-	const char*	id);	/* in: identifier to check.  it must belong
-				to charset my_charset_filename */
-
 /**********************************************************************
 Converts an identifier from my_charset_filename to UTF-8 charset. */
 uint
