@@ -1376,13 +1376,12 @@ dberr_t srv_start(bool create_new_db)
 
 	if (!srv_read_only_mode) {
 		if (srv_innodb_status) {
-
+			const size_t len = strlen(fil_path_to_mysql_datadir) +
+				20 + sizeof "/innodb_status.";
 			srv_monitor_file_name = static_cast<char*>(
-				ut_malloc_nokey(
-					strlen(fil_path_to_mysql_datadir)
-					+ 20 + sizeof "/innodb_status."));
+				ut_malloc_nokey(len));
 
-			sprintf(srv_monitor_file_name,
+			snprintf(srv_monitor_file_name, len,
 				"%s/innodb_status." ULINTPF,
 				fil_path_to_mysql_datadir,
 				static_cast<ulint>
