@@ -94,7 +94,7 @@ struct st_row_connect_config {
   char ssl_crl_path[FN_REFLEN];
   uint ssl_crl_path_length;
   uint connection_retry_interval;
-  ulong connection_retry_count;
+  ulonglong connection_retry_count;
   double heartbeat_interval;
   char *ignore_server_ids;
   uint ignore_server_ids_length;
@@ -153,10 +153,10 @@ protected:
     @param read_all         true if all columns are read.
   */
 
-  virtual int read_row_values(TABLE *table,
-                              unsigned char *buf,
-                              Field **fields,
-                              bool read_all);
+  int read_row_values(TABLE *table,
+                      unsigned char *buf,
+                      Field **fields,
+                      bool read_all) override;
 
   table_replication_connection_configuration();
 
@@ -168,9 +168,9 @@ public:
   static PFS_engine_table_share m_share;
   static PFS_engine_table* create();
   static ha_rows get_row_count();
-  virtual int rnd_next();
-  virtual int rnd_pos(const void *pos);
-  virtual void reset_position(void);
+  int rnd_next() override;
+  int rnd_pos(const void *pos) override;
+  void reset_position(void) override;
 
 };
 
