@@ -5,15 +5,15 @@ static void my_hasher_base31_hash_str(my_hasher_st *hasher, const uchar *key,
                                       size_t len)
 {
   for (const uchar *u= key; u < key + len; u++)
-    hasher->m_nr= ((uint) *u) * 31 + hasher->m_nr;
+    hasher->m_nr= hasher->m_nr * 31 + ((uint) *u);
 }
 
 static void my_hasher_base31_hash_byte(my_hasher_st *hasher, uchar byte)
 {
-  hasher->m_nr= ((uint) byte) * 31 + hasher->m_nr;
+  hasher->m_nr= hasher->m_nr * 31 + ((uint) byte);
 }
 
-static uint32 my_hasher_base31_finalize(my_hasher_st *hasher)
+static uint64_t my_hasher_base31_finalize(my_hasher_st *hasher)
 {
   return hasher->m_nr;
 }
@@ -22,7 +22,7 @@ static uint32 my_hasher_base31_finalize(my_hasher_st *hasher)
 my_hasher_st my_hasher_base31(void)
 {
   my_hasher_st tmp=
-    { 1, 4, 0, FALSE, my_hasher_base31_hash_str, my_hasher_base31_hash_byte,
-      my_hasher_base31_finalize, NULL };
+    { {.m_nr = 0}, FALSE, my_hasher_base31_hash_str, my_hasher_base31_hash_byte,
+      my_hasher_hash_num, my_hasher_base31_finalize, NULL };
   return tmp;
 }
